%% Emulation of FFTlab in Matlab
%% ===========================================================================

global u U			% time-domain and frequency-domain arrays
global t f N Nq filt handle

dt = 1.0;
n0 = 16;				% number of sample at zero time
t = [ (0:(n0)), (-n0+1):(-1) ]*dt;	% time sampling in FFT inputs;
					% times above n0 shown as negative

%t = [ 0:(2*n0) ]*dt;

N = length(t);		% number of time and frequency samples (should be even)

df = 1.0/dt/N;		% frequency sampling interval

Nq = fix(N/2);		% Nyquist frequency index

indNq = 1:Nq;		% range of indices to Nyquist frequency

f = [ (0:(Nq-1)), (-Nq):(-1) ]*df;  % frequency sampling in FFT outputs; 
				% frequencies above Nyquist shown as negative

%f = [ 0:(N-1) ]*df;

u = zeros(N,1);		% initial data

u(1) = 1;

U = fft(u);		% FFT transform

maxu = max(abs(u))+1.0e-8;	% peak values in u and U
maxU = max(abs(U))+1.0e-8;

% structure used for selecting shapes on subplots

select.subplot = 0;
select.xy = [];
 
fig = figure('Name','FFT Lab','Position',[100,100,1600,800],...
                  'menubar','none','toolbar','none','numbertitle','off'); % figure properties
clf

% ui buttons below seem to conflict with ginput(), so using only ginput() with keyboard keys.
% Perhaps I could ButtonDownFcn callback in subplot(), but this looks more complicated.

%ui_clear = uicontrol('style','pushbutton','string',"Clear",'units','normalized', ...
%            'position',[0.05, 0.02, 0.04, 0.04],'backgroundcolor',[.8, .8, .8],'Callback',@callback_clear);

%uicontrol('style','pushbutton','string',"Exit",'units','normalized', ...
%            'position',[0.55, 0.02, 0.05, 0.05],'backgroundcolor',[.8, .8, .8],'Callback',@callback_exit);

uicontrol('style','text','string','Keyboard commands: a - apply filter, c - clear data, f - clear filter, q - exit.      Mouse buttons: MB1 - set range, MB3 - set one point',...
		'HorizontalAlignment','center','units','normalized', ...
            	'position',[0.2 0.01 0.6 0.03]);

% arrays containing lists of currently plotted objects in each subplot

handle.plt1 = [];
handle.plt2 = [];
handle.plt3 = [];
handle.plt4 = [];
handle.plt5 = [];
handle.plt6 = [];
handle.filt = [];

filt = [];		% filter

while 1

	[ ax maxu maxU ] = fftlab_display();		% display everything

	[x,y,buttonNumber] = ginput(1);

	[~, axnum ] = ismember(gca,ax);		% axnum = number of subplot clicked

	% scale y to the unnormalized units of u or U

	switch axnum
		case { 1, 4 }		% clicking on time-domain subplots
			y = y * maxu;

		case { 2, 5 }		% clicking on frequeny-domain subplots
			y = y * maxU;
	end

	switch buttonNumber

	  case 1		% left mouse button - select ranges of time or frequency

		% if clicked on the same subplot, add to selection, otherwise start new selection 

		if axnum == select.subplot	
			select.xy = [ select.xy; x, y ];

			% sort by x

			[~, ind ] = sort(select.xy(:,1),1);		

			select.xy = select.xy(ind,:);

		else
			select.subplot = axnum;
			select.xy = [ x, y ];
		end 

%select
		if ~isempty(select.xy)		% apply selection to data 
		  switch axnum
			case 1	% setting real part in time domain

				u = selection(t,u,select.xy,true);
				U = fft(u);

			case 2	% setting imaginary part in time domain

				u = selection(t,u,select.xy,false);
				U = fft(u);

			case 3	% setting real part in frequency domain

				U = selection(f,U,select.xy,true);
				u = ifft(U);

			case 4	% setting imaginary part in frequency domain

				U = selection(f,U,select.xy,false);
				u = ifft(U);

			case 5	% setting zero-phase filter

				filt = transpose(selection(f,zeros(1,length(U)),select.xy,true));
				filt = filt(indNq,1);
		  end		
		end

	  case 3		% right mouse button - select individual points 

		switch axnum
			case 1	% setting real part in time domain

				u = selection1(t,u,[x,y],true);
				U = fft(u);

			case 2	% setting imaginary part in time domain

				u = selection1(t,u,[x,y],false);
				U = fft(u);

			case 3	% setting real part in frequency domain

				U = selection1(f,U,[x,y],true);
				u = ifft(U);

			case 4	% setting imaginary part in frequency domain

				U = selection1(f,U,[x,y],false);
				u = ifft(U);	
		end

	  case 'a'		% apply filter

		% multiply the positive- and negative-frequency parts

		U(indNq) = U(indNq).*filt;

		ind = N + 1 - indNq;		% negative-frequency part	

		U(ind) = U(ind).*filt;

		u = ifft(U);

	  case 'c'		% clear

		u = zeros(size(u));
		U = zeros(size(U));

		select.subplot = 0;
		select.xy = [];

	  case 'f'		% clear filter

		clear_display(handle.filt);

		handle.filt = [];
		filt = [];

		select.subplot = 0;	% also clear selection
		select.xy = [];

	  case 'q'

        query = questdlg('Exit?', 'FFT Lab', 'YES', 'NO', 'NO');

        if strcmpi(query,'yes')
            close(fig)      % close the figure
            break
        end

	  case 'p'

		    msgbox("Image export not implemented. Use Matlab menu.");

	end	% switch

end	% big endless while loop
