%% Display current arrays u and U
%% Returns:
%%	ax - array of axes handles of the six subplots
%%	maxu - amplitude corresponding to Y = 1 in time-domain plots 
%%	maxU - amplitude corresponding to Y = 1 in frequency-domain plots 
%% ===========================================================================

function [ ax maxu maxU ]= fftlab_display()

global u U			% time-domain and frequency-domain arrays
global t f Nq filt handle 

	maxt = max(abs(t))*1.1;
	maxf = max(abs(f))*1.1;
	maxu = max(abs(u));
	maxU = max(abs(U));

	if maxu < 1.0e-8	% for zero amplitudes, fix max to equal 1
		maxu = 1.0;
	end

	if maxU < 1.0e-8
		maxU = 1.0;
	end

	tickt_inc = 5.0;		% tick marks on time axes

	tickt = floor(maxt/tickt_inc)*tickt_inc;

	tickt = [-tickt:tickt_inc:tickt];

	Up = U(1:Nq);		% positive-frequency part
	fp = f(1:Nq);

	% time-domain plots (left column)

	subplot(2,3,1)
	ax = gca;		% store the list of axes

	clear_display(handle.plt1);
	handle.plt1 = plot_bars(t,real(u)/maxu);

	xlim([-maxt, maxt]);
	ylim([-1.1, 1.1]);

	grid on
	set(gca,'Xtick',tickt);

	xlabel('Time (ms)')
	ylabel('Re(u)/peak(|u|)')
	title(sprintf('Time domain, real part, scale = %1.3f',maxu))

	subplot(2,3,4)
	ax = [ ax, gca ];		% add to the list of axes

	clear_display(handle.plt4);
	handle.plt4 = plot_bars(t,imag(u)/maxu);

	xlim([-maxt, maxt]);
	ylim([-1.1, 1.1]);

	grid on
	set(gca,'Xtick',tickt);

	xlabel('Time (ms)')
	ylabel('Im(u)/peak(|u|)')
	title('Time domain, imaginary part')

	% frequency-domain plots (middle column)

	subplot(2,3,2)
	ax = [ ax, gca ];		% add to the list of axes

	clear_display(handle.plt2);
	handle.plt2 = plot_bars(f,real(U)/maxU);

	xlim([-maxf, maxf]);
	ylim([-1.1, 1.1]);

	grid on
	xlabel('Frequency (kHz)')
	ylabel('Re(U)/peak(|U|)')
	title(sprintf('Frequency domain, real part, scale = %1.3f',maxU))

	subplot(2,3,5)
	ax = [ ax, gca ];		% add to the list of axes

	clear_display(handle.plt5);
	handle.plt5 = plot_bars(f,imag(U)/maxU);

	xlim([-maxf, maxf]);
	ylim([-1.1, 1.1]);

	grid on
	xlabel('Frequency (kHz)')
	ylabel('Re(U)/peak(|U|)')
	title('Frequency domain, imaginary part')

	% spectra plots (right column)

	subplot(2,3,3)
	hold on
	ax = [ ax, gca ];		% add to the list of axes

	clear_display(handle.filt);

	if isempty(filt)
		handle.filt = [];
	else
		handle.filt = line(fp,abs(filt(1:Nq)),'Color','g');
	end

	clear_display(handle.plt3);
	handle.plt3 = line(fp,abs(Up)/maxU,'Marker','x','Color','r');

	xlim([0, maxf]);
	ylim([0, 1.1]);

	grid on
	xlabel('Frequency (kHz)')
	ylabel('|U|/peak(|U|)')
	title('Amplitude spectrum')

	subplot(2,3,6)

	ax = [ ax, gca ];		% add to the list of axes

	clear_display(handle.plt6);
	handle.plt6 = line(fp,angle(Up)*180/pi,'Marker','x','Color','r');

	xlim([0, maxf]);
	ylim([-180,180]);

	grid on
	xlabel('Frequency (kHz)')
	ylabel('Arg(U) (deg)')
	title('Phase spectrum')
end

