%% Helper function for FFTlab
%% First, the values of function y(x) are  set equal zero and then,
%% within the range of values covered by the first column in 'xy',
%% y(x) is set by interpolating the second colum of 'xy'
%%
%% Parameters:
%%      x - argument of the function being set (time or frequency)
%%	y - value of the function being set 
%%	xy - matrix with two columns. First column contains x values for the picks
%%		and the second column contains values y(x) set at these picks 
%%	real_part - if true, setting the real part of y, otherwise setting the imaginary part
%%
%% Returns:
%%	ynew - edited complex-valued array 'y'

function ynew = selection(x,y,xy,real_part)

	mask = zeros(size(y));

	dx = (x(:,2) - x(:,1))*0.5;	% tolerance

	ind = find( x > (min(xy(:,1) - dx)) & x < (max(xy(:,1) + dx) ) );	% range of indices to edit

%xy
%ind

	if ~isempty(ind) 
	  if size(xy,1) > 1
		mask(ind) = interp1(xy(:,1),xy(:,2),x(ind),'linear','extrap');	% linear interpolation
	  else
		[x0,n] = min(abs(xy(:,1) - x(ind)));

		mask(ind) = xy(n,2);		% nearest point
	  end
	end

	if real_part
		ynew = mask + 1i*imag(y);
	else
		ynew = real(y) + 1i*mask;
	end
end

